#ifndef CGUL_LIST_NODE_H
#define CGUL_LIST_NODE_H

/**
 * This class implements a linked list node which holds one value.
 *
 * @file
 * @author Paul Serice
 * @brief linked list node
 */

#include "cgul_common.h"
#include "cgul_exception.h"

CGUL_BEGIN_C

/** Opaque pointer to a <code>cgul_list_node</code> instance. */
typedef struct cgul_list_node* cgul_list_node_t;

/*
 * The cgul_list_node constructor and destructor are private.  Only
 * the <code>cgul_list</code> should create or destroy its nodes.
 */

/**
 * Return the value stored in the node.
 *
 * @param[in] cex c-style exception
 * @param[in] n <code>cgul_list_node</code> instance
 * @return value stored in <code>n</code>
 */
extern CGUL_EXPORT void*
cgul_list_node__get_value(cgul_exception_t* cex,
                          cgul_list_node_t n);

/**
 * Set the value to be stored in the node.
 *
 * @param[in] cex c-style exception
 * @param[in] n <code>cgul_list_node</code> instance
 * @param[in] value value to be stored in the node
 */
extern CGUL_EXPORT void
cgul_list_node__set_value(cgul_exception_t* cex,
                          cgul_list_node_t n,
                          const void* value);

/**
 * Return the previous node.  You can use this method to iterate over
 * the list.  When there are no more nodes, <code>NULL</code> is
 * returned.  You can use <code>cgul_list__get_back()</code> to get
 * the last node in a <code>cgul_list</code> in order to start
 * iterating.  A <code>cgul_list_node__set_prev()</code> exists, but
 * it is private.
 *
 * @param[in] cex c-style exception
 * @param[in] n <code>cgul_list_node</code> instance
 * @return previous node
 */
extern CGUL_EXPORT cgul_list_node_t
cgul_list_node__get_prev(cgul_exception_t* cex,
                         cgul_list_node_t n);

/**
 * Return the next node.  You can use this method to iterate over
 * the list.  When there are no more nodes, <code>NULL</code> is
 * returned.  You can use <code>cgul_list__get_front()</code> to get
 * the first node in a <code>cgul_list</code> in order to start
 * iterating.  A <code>cgul_list_node__set_next()</code> exists, but
 * it is private.
 *
 * @param[in] cex c-style exception
 * @param[in] n <code>cgul_list_node</code> instance
 * @return next node
 */
extern CGUL_EXPORT cgul_list_node_t
cgul_list_node__get_next(cgul_exception_t* cex,
                         cgul_list_node_t n);


CGUL_END_C

#endif  /*  CGUL_LIST_NODE_H  */
