#ifndef CGUL_LOG2_CXX_H
#define CGUL_LOG2_CXX_H

/**
 * C++ wrapper for <code>cgul_log2</code>.
 *
 * @file
 * @author Paul Serice
 * @brief C++ wrapper for <code>cgul_log2</code>
 */

#include "cgul_exception_cxx.h"
#include "cgul_log2.h"

/**
 * This class provides the C++ bindings for <code>cgul_log2</code>.
 * The main purpose of this class is to convert the C-style function
 * calls and exception handling in <code>cgul_log2</code> into
 * C++-style function calls and exception handling.
 *
 * @brief C++ bindings for <code>cgul_log2</code>
 */
class cgul_log2_cxx {

public:

    /**
     * Return the floor of the base-2 logarithm of the number
     * <code>n</code>.  If <code>n</code> is zero, an exception is
     * thrown.
     *
     * @param[in] n number
     * @return floor(log2(n))
     */
    static int
    floor(unsigned long int n)
    {
        int rv = 0;
        cgul_exception_t local = NULL;
        rv = cgul_log2__floor(&local, n);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        return rv;
    }

    /**
     * Return the ceiling of the base-2 logarithm of the number
     * <code>n</code>.  If <code>n</code> is zero, an exception is
     * thrown.
     *
     * @param[in] n number
     * @return ceil(log2(n))
     */
    static int
    ceil(unsigned long int n)
    {
        int rv = 0;
        cgul_exception_t local = NULL;
        rv = cgul_log2__ceil(&local, n);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        return rv;
    }

};

#endif  /*  CGUL_LOG2_CXX_H  */
