#include "cgul_block_reader_p.h"

void
cgul_block_reader__delete(cgul_block_reader_t br)
{
    /* Polymorphically delete the object. */
    if (br && br->vptr && br->vptr->virtual_delete){
        br->vptr->virtual_delete(br);
    }
}

size_t
cgul_block_reader__read_next_block(cgul_exception_t* cex,
                                   cgul_block_reader_t br,
                                   void* ptr,
                                   size_t block_size)
{
    size_t rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Polymorphically read blocks of data. */
    if (br && br->vptr && br->vptr->virtual_fread){
        rv = br->vptr->virtual_fread(cex, br, ptr, block_size);
        if (*cex) {
            goto out;
        }
    }

 out:
    return rv;
}

const char*
cgul_block_reader__get_presentation_name(cgul_exception_t* cex,
                                         cgul_block_reader_t br)
{
    const char* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Polymorphically get the presentation name. */
    if (br && br->vptr && br->vptr->virtual_get_presentation_name) {
        rv = br->vptr->virtual_get_presentation_name(cex, br);
        if (*cex) {
            goto out;
        }
    }

 out:
    return rv;
}

void
cgul_block_reader__set_presentation_name(cgul_exception_t* cex,
                                         cgul_block_reader_t br,
                                         const char* presentation_name)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Polymorphically set the presentation name. */
    if (br && br->vptr && br->vptr->virtual_set_presentation_name) {
        br->vptr->virtual_set_presentation_name(cex, br, presentation_name);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}
