#ifndef CGUL_CRC16_INCR_P_H
#define CGUL_CRC16_INCR_P_H

#include "cgul_crc16_incr.h"
#include "cgul_crc16.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_crc16_incr</code> class.
 */
struct cgul_crc16_incr {

    /**
     * Incremental CRC-16 value.  This is the object that is directly
     * updated.  When a final CRC-16 value is needed, we simply assign
     * this object to <code>finalcrc</code> and call
     * <code>cgul_crc16__crc()</code> on the <code>finalcrc</code>
     * object thereby avoiding destruction of the CRC-16 state that
     * we've worked so hard to build up.
     */
    cgul_crc16_t incr_crc;

    /**
     * Final CRC-16 value.  When a final CRC-16 value is needed, we
     * simply assign <code>incrcrc</code> to this object and call
     * <code>cgul_crc16__crc()</code> on this object thereby avoiding
     * destruction of the CRC-16 state in <code>incrcrc</code> that
     * we've worked so hard to build up.
     */
    cgul_crc16_t final_crc;

};

CGUL_END_C

#endif  /*  CGUL_CRC16_INCR_P_H  */
