#ifndef CGUL_HEX_DUMP_P_H
#define CGUL_HEX_DUMP_P_H

#include "cgul_common.h"
#include "cgul_hex_dump.h"
#include "cgul_list.h"

CGUL_BEGIN_C

struct cgul_hex_dump {

    /**
     * The list of strings is not one string per line.  It is one
     * string per call to cgul_hex_dump__append().  As such, the new
     * line characters are added in-line as cgul_hex_dump__append()
     * process the input.  We need to distinguish the case when a new
     * line of data is appended immediately after a call to
     * cgul_hex_dump__clear() in which case no new line should be
     * appended.
     */
    int suppress_new_line;

    /* Inner loop counter.  It makes sure we have only 16 bytes per line. */
    size_t j;

    /** The left side of the output keeps a running count of the bytes. */
    unsigned int byte_count;

    /**
     * Keep a running total of the combined lengths of all the
     * strings on the <code>strings</code> linked list.
     */
    size_t total_length;

    /**
     * You need to be able to remember the printable characters across
     * calls to <code>cgul_hex_dump__append()</code> because the
     * strings you store in the <code>strings</code> linked list will
     * (generally) not have the printable characters for the last
     * line.  Instead, we wait for the next call to
     * <code>cgul_hex_dump__append()</code> so that we will have
     * enough characters to complete the last line.  Then, we will
     * append the printable_characters string.
     */
    cgul_string_t printable_chars;

    /**
     * For efficiency, we allocate a new <code>cgul_string</code> for
     * each call the user makes to
     * <code>cgul_hex_dump__append()</code> and set its capacity so
     * that it can handle the entire call without having to resize
     * itself.  When the user calls
     * <code>cgul_hex_dump__get_value()</code> we chain all the
     * strings together hopefully resulting in the smallest average
     * number of <code>realloc()</code> calls.
     */
    cgul_list_t strings;

};

CGUL_END_C

#endif  /*  CGUL_HEX_DUMP_P_H  */
