#ifndef CGUL_MULTIMAP_P_H
#define CGUL_MULTIMAP_P_H

#include "cgul_common.h"
#include "cgul_multimap.h"
#include "cgul_multimap_node_p.h"
#include "cgul_rbtree.h"

CGUL_BEGIN_C

/** Data encapsulated by the <code>cgul_multimap</code> class. */
struct cgul_multimap {

    /**
     * Size of the multimap.  This is the count of the number of nodes
     * held by the tree.  This is the count of the number of values
     * held held in the multimap.  If you need to know the number
     * keys, just query the size of the <code>cgul_rbtree</code>.
     */
    unsigned long int size;

    /**
     * Underlying red-black tree.
     */
    cgul_rbtree_t tree;

    /**
     *  The oldest node in the multimap.  This is the first node that
     *  was inserted.
     */
    cgul_multimap_node_t oldest;


    /**
     *  The youngest node in the multimap.  This is the last node that
     *  was inserted.
     */
    cgul_multimap_node_t youngest;

};

/**
 * Data needed for <code>cgul_multimap__clear_cb()</code>.
 */
typedef struct cgul_multimap__clear_cb_data {

    /** The <code>cgul_cache</code> the client is using for caching keys. */
    cgul_cache_t keys_cache;

    /** The <code>cgul_cache</code> the client is using for caching values. */
    cgul_cache_t values_cache;

    /** Return value indicating whether any nodes have been removed. */
    int rv;

} *cgul_multimap__clear_cb_data_t;

CGUL_END_C

#endif  /*  CGUL_MULTIMAP_P_H  */
