#ifndef CGUL_RANDOM_BBS_P_H
#define CGUL_RANDOM_BBS_P_H

#include "cgul_common.h"
#include "cgul_big_integer.h"
#include "cgul_random_bbs.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_random_bbs</code> class.
 */
struct cgul_random_bbs {

    /**
     * Current value.
     */
    cgul_big_integer_t curr;

    /**
     * m the modulus in the Blum-Blum-Shub generator.  Because it is
     * prime and not a power of 2, it really cannot be used for much
     * of anything else.  Instead, use <code>max</code> which is an
     * exclusive, upper bound on the pseudo-random numbers returned by
     * this class.
     */
    cgul_big_integer_t m;

    /**
     * This is the scaled reciprocal of <code>m</code>.  When the code
     * needs to divide by "m", it multiplies by the reciprocal instead.
     */
    cgul_big_integer_t m_reciprocal;

    /**
     * This is the power of 2 to which <code>m_reciprocal</code> is
     * scaled.
     */
    size_t m_scale;

    /**
     * Mask used to limit the number of bits in <code>curr</code> that
     * are used.
     */
    cgul_big_integer_t mask;

    /**
     * This variable is used as the exclusive, upper bound on the
     * pseudo-random numbers returned by this class.
     */
    cgul_big_integer_t max;

    /**
     * This is the scaled reciprocal of <code>max</code>.  When the code
     * needs to divide by "max", it multiplies by the reciprocal instead.
     */
    cgul_big_integer_t max_reciprocal;

    /**
     * This is the power of 2 to which <code>max_reciprocal</code> is
     * scaled.
     */
    size_t max_scale;

    /**
     * Scratch.
     */
    cgul_big_integer_t tmp;

};

CGUL_END_C

#endif  /*  CGUL_RANDOM_BBS_P_H  */
