#include "cgul_libc.h"
#include "cgul_split.h"
#include "cgul_strtok.h"

void
cgul_split(cgul_exception_t* cex,
           const char* s,
           const char* separators,
           int allow_empty_parts,
           cgul_vector_t parts)
{
    int roll_back_on_failure = 0;
    size_t i = 0;
    unsigned long int parts_size_init = 0;
    unsigned long int parts_size_curr = 0;
    const char* part = NULL;
    char* tmp = NULL;
    cgul_strtok_t tok = NULL;
    cgul_exception_t local = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(parts);

    /* Remember the initial size of "parts" in case an exception is
     * thrown and this function has to roll back its changes. */
    parts_size_init = cgul_vector__get_size(cex, parts);

    /* Now that parts_size_init is set, flag that it is safe to roll
     * back insertions. */
    roll_back_on_failure = 1;

    /* Allocate the tokenizer. */
    tok = cgul_strtok__new(cex);
    if (*cex) {
        goto out;
    }

    /* Initialize the tokenizer. */
    cgul_strtok__set_value(cex, tok, s);
    if (*cex) {
        goto out;
    }

    /* Iterate over the tokens. */
    for (;;) {

        /* Get the next token. */
        part = cgul_strtok__get_next(cex, tok, separators, allow_empty_parts);
        if (!part) {
            break;
        }

        /* Duplicate the token. */
        tmp = cgul_libc__strdup(cex, part);

        /* Add the token to the vector. */
        cgul_vector__push_back(cex, parts, tmp);

        /* Check for exceptions. */
        if (*cex) {
            goto out;
        }

        /* Take ownership of the token. */
        tmp = NULL;
    }

 out:

    /* If an exception is thrown, remove all the strings that this
     * function inserted into the "parts" vector. */
    if (*cex && parts && roll_back_on_failure) {
        parts_size_curr = cgul_vector__get_size(&local, parts);
        for (i = parts_size_curr ; i > parts_size_init ; --i) {
            /* Remove and free the string that was inserted. */
            tmp = (char*)cgul_vector__pop_back(&local, parts);
            free(tmp);
        }
    }

    /* Clean up. */
    if (tok) {
        cgul_strtok__delete(tok);
    }
    if (tmp) {
        free(tmp);
    }

    return;
}
