#include "cgul_hstring_p.h"
#include "cgul_string_p.h"
#include "cgul_unicode.h"

void
cgul_unicode__mbstowcs(cgul_exception_t* cex,
                       const char* utf8,
                       size_t utf8_length,
                       int skip_bom,
                       cgul_wstring_t utf32)
{
    size_t index = 0;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf8);
    assert(utf32);

    /* Iterate over each character in the utf8 string. */
    while (index < utf8_length) {

        /* Convert the UTF-8 character sequence to a UTF-32 character. */
        wc = cgul_unicode__mbtowc(cex, utf8, utf8_length, &index);
        if (*cex) {
            goto out;
        }

        /* Skip BOM. */
        if ((index == 3) && skip_bom && (wc == CGUL_WCHAR__BOM)) {
            continue;
        }

        /* Append to utf32. */
        cgul_wstring__append_char(cex, utf32, wc);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

void
cgul_unicode__wcstombs(cgul_exception_t* cex,
                       const cgul_wchar_t* utf32,
                       size_t utf32_length,
                       int skip_bom,
                       cgul_string_t utf8)
{
    size_t i = 0;
    char buf[CGUL_WCHAR__MB_LEN_MAX + 1] = {'\0'};

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf32);
    assert(utf8);

    /* Iterate over each character in the utf32 string. */
    for (i = 0 ; i < utf32_length ; ++i) {

        /* Skip BOM. */
        if ((i == 0) && skip_bom && (utf32[0] == CGUL_WCHAR__BOM)) {
            continue;
        }

        /* Convert the UTF-32 character to a UTF-8 sequence. */
        cgul_unicode__wctomb(cex, utf32[i], buf);
        if (*cex) {
            goto out;
        }

        /* Append to utf8. */
        cgul_string__append(cex, utf8, buf);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

void
cgul_unicode__mbstohcs(cgul_exception_t* cex,
                       const char* utf8,
                       size_t utf8_length,
                       int skip_bom,
                       cgul_hstring_t utf16)
{
    size_t index = 0;
    cgul_hchar_t buf[3] = {0};

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf8);
    assert(utf16);

    /* Iterate over each character in the utf8 string. */
    while (index < utf8_length) {

        /* Convert the UTF-8 character sequence to a UTF-16 sequence. */
        cgul_unicode__mbtohc(cex, utf8, utf8_length, &index, buf);
        if (*cex) {
            goto out;
        }

        /* Skip BOM. */
        if ((index == 3) && skip_bom && (buf[0] == CGUL_HCHAR__BOM)) {
            continue;
        }

        /* Append to utf16. */
        cgul_hstring__append(cex, utf16, buf);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

void
cgul_unicode__hcstombs(cgul_exception_t* cex,
                       const cgul_hchar_t* utf16,
                       size_t utf16_length,
                       int skip_bom,
                       cgul_string_t utf8)
{
    size_t index = 0;
    char buf[CGUL_WCHAR__MB_LEN_MAX + 1] = {'\0'};

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf16);
    assert(utf8);

    /* Iterate over each character in the utf16 string. */
    while (index < utf16_length) {

        /* Convert the UTF-16 character sequence to a UTF-8 sequence. */
        cgul_unicode__hctomb(cex, utf16, utf16_length, &index, buf);
        if (*cex) {
            goto out;
        }

        /* Skip BOM. */
        if ((index == 1) && skip_bom && (utf16[0] == CGUL_HCHAR__BOM)) {
            continue;
        }

        /* Append to utf8. */
        cgul_string__append(cex, utf8, buf);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

void
cgul_unicode__hcstowcs(cgul_exception_t* cex,
                       const cgul_hchar_t* utf16,
                       size_t utf16_length,
                       int skip_bom,
                       cgul_wstring_t utf32)
{
    size_t index = 0;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf16);
    assert(utf32);

    /* Iterate over each character in the utf16 string. */
    while (index < utf16_length) {

        /* Convert the UTF-16 character sequence to a UTF-32 character. */
        wc = cgul_unicode__hctowc(cex, utf16, utf16_length, &index);
        if (*cex) {
            goto out;
        }

        /* Skip BOM. */
        if ((index == 1) && skip_bom && (wc == CGUL_WCHAR__BOM)) {
            continue;
        }

        /* Append to utf32. */
        cgul_wstring__append_char(cex, utf32, wc);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

void
cgul_unicode__wcstohcs(cgul_exception_t* cex,
                       const cgul_wchar_t* utf32,
                       size_t utf32_length,
                       int skip_bom,
                       cgul_hstring_t utf16)
{
    size_t i = 0;
    cgul_hchar_t buf[3] = {0};

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf32);
    assert(utf16);

    /* Iterate over each character in the utf32 string. */
    for (i = 0 ; i < utf32_length ; ++i) {

        /* Skip BOM. */
        if ((i == 0) && skip_bom && (utf32[0] == CGUL_WCHAR__BOM)) {
            continue;
        }

        /* Convert the UTF-32 character to a UTF-16 sequence. */
        cgul_unicode__wctohc(cex, utf32[i], buf);
        if (*cex) {
            goto out;
        }

        /* Append to utf16. */
        cgul_hstring__append(cex, utf16, buf);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

cgul_wchar_t
cgul_unicode__mbtowc(cgul_exception_t* cex,
                     const char* utf8,
                     size_t utf8_length,
                     size_t* index)
{
    cgul_wchar_t rv = CGUL_WCHAR__NUL;
    cgul_wchar_t b0 = CGUL_WCHAR__NUL;
    cgul_wchar_t b1 = CGUL_WCHAR__NUL;
    cgul_wchar_t b2 = CGUL_WCHAR__NUL;
    cgul_wchar_t b3 = CGUL_WCHAR__NUL;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;
    size_t i = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf8);
    assert(index);

    /*
     * NOTE: Section 3.9 of the Unicode 8.0 standard gives the basic
     * rules for encoding UTF-8 where the first byte passes through
     * 7-bit ASCII and encodes the number of subsequent bytes and
     * where all the subsequent bytes start with 0x10...
     *
     * It further adds the requirements that code points use the
     * shortest form and disallows code points reserved for UTF-16
     * surrogate pairs.  Fortunately, the standard has a table that
     * says exactly what is allowed:
     *
     * |--------------------+---------+---------+---------+---------|
     * | Code Points        | byte[0] | byte[1] | byte[2] | byte[3] |
     * |--------------------+---------+---------+---------+---------|
     * | U+0000..U+007F     | 00..7F  |         |         |         |
     * |--------------------+---------+---------+---------+---------|
     * | U+0080..U+07FF     | C2..DF  | 80..BF  |         |         |
     * |--------------------+---------+---------+---------+---------|
     * | U+0800..U+0FFF     | E0      | A0..BF  | 80..BF  |         |
     * | U+1000..U+CFFF     | E1..EC  | 80..BF  | 80..BF  |         |
     * | U+D000..U+D7FF     | ED      | 80..9F  | 80..BF  |         |
     * | U+E000..U+FFFF     | EE..EF  | 80..BF  | 80..BF  |         |
     * |--------------------+---------+---------+---------+---------|
     * | U+10000..U+3FFFF   | F0      | 90..BF  | 80..BF  | 80..BF  |
     * | U+40000..U+FFFFF   | F1..F3  | 80..BF  | 80..BF  | 80..BF  |
     * | U+100000..U+10FFFF | F4      | 80..8F  | 80..BF  | 80..BF  |
     * |--------------------+---------+---------+---------+---------|
     *
     * [The "byte[1]" column must be read very carefully!]
     */

    /* Copy "*index" to "i".  This way we don't alter "*index" if a
     * failure occurs. */
    i = *index;

    /* Bounds check "i". */
    if (i >= utf8_length) {
        *cex = cgul_exception__new("cgul_unicode__mbtowc(): "
                                   "index out of bounds: %zu >= %zu",
                                   i, utf8_length);
        goto out;
    }

    /* The first encoded byte tells us how how many bytes total
     * are used to encode this character and adds a pre-determined
     * number of bits to the decoded value. */
    b0 = (unsigned char)utf8[i];
    if ((b0 & 0x80) == 0) {

        /* The first byte looks like "0xxxxxxx" which means there
         * is one byte total in this character's encoding. */

        /* 7-bit ASCII passes through. */
        wc = b0;
        i += 1;

    } else if ((b0 & 0xe0) == 0xc0) {

        /* The first byte looks like "110xxxxx" which means there
         * are two bytes total in this character's encoding. */

        /* Bounds check "i + 1". */
        if (i + 1 >= utf8_length) {
            *cex = cgul_exception__new("cgul_unicode__mbtowc(): "
                                       "index out of bounds: %zu >= %zu",
                                       i + 1, utf8_length);
            goto out;
        }

        /* Get the remaining bytes for this encoding. */
        b1 = (unsigned char)utf8[i + 1];

        /* Range check the bytes.  (See table above.) */
        if ((0xC2 <= b0) && (b0 <= 0xDF) &&
            (0x80 <= b1) && (b1 <= 0xBF))
        {
            /* Decode */
            wc = ((b0 & 0x1f) << 6) | (b1 & 0x3f);
            i += 2;

        } else {

            /* Error. */
            *cex = cgul_exception__new(
                "cgul_unicode__mbtowc(): ill-formed UTF-8: "
                "invalid encoding start at index %zu",
                i);
            goto out;

        }

    } else if ((b0 & 0xf0) == 0xe0) {

        /* The first byte looks like "1110xxxx" which means there
         * are three bytes total in this character's encoding. */

        /* Bounds check "i + 2". */
        if (i + 2 >= utf8_length) {
            *cex = cgul_exception__new("cgul_unicode__mbtowc(): "
                                       "index out of bounds: %zu >= %zu",
                                       i + 2, utf8_length);
            goto out;
        }

        /* Get the remaining bytes for this encoding. */
        b1 = (unsigned char)utf8[i + 1];
        b2 = (unsigned char)utf8[i + 2];

        /* Range check the bytes.  (See table above.) */
        if (/* row 3 */
            ((b0 == 0xE0) &&
             ((0xA0 <= b1) && (b1 <= 0xBF)) &&
             ((0x80 <= b2) && (b2 <= 0xBF))) ||
            /* row 4 */
            (((0xE1 <= b0) && (b0 <= 0xEC)) &&
             ((0x80 <= b1) && (b1 <= 0xBF)) &&
             ((0x80 <= b2) && (b2 <= 0xBF))) ||
            /* row 5 */
            ((b0 == 0xED) &&
             ((0x80 <= b1) && (b1 <= 0x9F)) &&
             ((0x80 <= b2) && (b2 <= 0xBF))) ||
            /* row 6 */
            (((0xEE <= b0) && (b0 <= 0xEF)) &&
             ((0x80 <= b1) && (b1 <= 0xBF)) &&
             ((0x80 <= b2) && (b2 <= 0xBF))))
        {
            /* Decode */
            wc =  ((b0 & 0x0f) << 12)
                | ((b1 & 0x3f) << 6)
                |  (b2 & 0x3f);
            i += 3;

        } else {

            /* Error. */
            *cex = cgul_exception__new(
                "cgul_unicode__mbtowc(): ill-formed UTF-8: "
                "invalid encoding start at index %zu",
                i);
            goto out;
        }

    } else if ((b0 & 0xf8) == 0xf0) {

        /* The first byte looks like "11110xxx" which means there
         * are four bytes total in this character's encoding. */

        /* Bounds check "i + 3". */
        if (i + 3 >= utf8_length) {
            *cex = cgul_exception__new("cgul_unicode__mbtowc(): "
                                       "index out of bounds: %zu >= %zu",
                                       i + 3, utf8_length);
            goto out;
        }

        /* Get the remaining bytes for this encoding. */
        b1 = (unsigned char)utf8[i + 1];
        b2 = (unsigned char)utf8[i + 2];
        b3 = (unsigned char)utf8[i + 3];

        /* Range check the bytes.  (See table above.) */
        if (/* row 7 */
            ((b0 == 0xF0) &&
             ((0x90 <= b1) && (b1 <= 0xBF)) &&
             ((0x80 <= b2) && (b2 <= 0xBF)) &&
             ((0x80 <= b3) && (b3 <= 0xBF))) ||
            /* row 8 */
            (((0xF1 <= b0) && (b0 <= 0xF3)) &&
             ((0x80 <= b1) && (b1 <= 0xBF)) &&
             ((0x80 <= b2) && (b2 <= 0xBF)) &&
             ((0x80 <= b3) && (b3 <= 0xBF))) ||
            /* row 9 */
            ((b0 == 0xF4) &&
             ((0x80 <= b1) && (b1 <= 0x8F)) &&
             ((0x80 <= b2) && (b2 <= 0xBF)) &&
             ((0x80 <= b3) && (b3 <= 0xBF))))
        {
            /* Decode */
            wc =  ((b0 & 0x07) << 18)
                | ((b1 & 0x3f) << 12)
                | ((b2 & 0x3f) << 6)
                |  (b3 & 0x3f);
            i += 4;

        } else {

            /* Error. */
            *cex = cgul_exception__new(
                "cgul_unicode__mbtowc(): ill-formed UTF-8: "
                "invalid encoding start at index %zu",
                i);
            goto out;

        }

    } else {

        /* The first byte is invalid. */
        *cex = cgul_exception__new(
            "cgul_unicode__mbtowc(): ill-formed UTF-8: "
            "invalid encoding start at index %zu",
            i);
        goto out;

    }

    /* Success */
    rv = wc;
    *index = i;

 out:
    return rv;
}

size_t
cgul_unicode__wctomb(cgul_exception_t* cex,
                     const cgul_wchar_t utf32,
                     char* buf)
{
    size_t rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(buf);

    /* Make sure utf32 is not in the range reserved for UTF-16
     * surrogate pairs. */
    if ((utf32 >= 0xd800) && (utf32 <= 0xdfff)) {
        *cex = cgul_exception__new(
                   "cgul_unicode__wctomb: "
                   "character in range reserved for "
                   "UTF-16 surrogate pairs: %#0lx",
                   (unsigned long)utf32);
        goto out;
    }

    /* Encode as one byte. */
    if (utf32 <= 0x7f) {
        buf[0] = (unsigned char)utf32;
        buf[1] = '\0';
        rv = 1;

    /* Encode as two bytes. */
    } else if (utf32 <= 0x7ff) {
        buf[0] = 0xc0 | ((unsigned char)(utf32 >> 6));
        buf[1] = 0x80 | (((unsigned char)(utf32)) & 0x3f);
        buf[2] = '\0';
        rv = 2;

    /* Encode as three bytes. */
    } else if (utf32 <= 0xffff) {
        buf[0] = 0xe0 | ((unsigned char)(utf32 >> 12));
        buf[1] = 0x80 | (((unsigned char)(utf32 >> 6)) & 0x3f);
        buf[2] = 0x80 | (((unsigned char)utf32) & 0x3f);
        buf[3] = '\0';
        rv = 3;

    /* Encode as four bytes. */
    } else if (utf32 <= 0x10ffff) {
        buf[0] = 0xf0 | ((unsigned char)(utf32 >> 18));
        buf[1] = 0x80 | (((unsigned char)(utf32 >> 12)) & 0x3f);
        buf[2] = 0x80 | (((unsigned char)(utf32 >> 6)) & 0x3f);
        buf[3] = 0x80 | (((unsigned char)utf32) & 0x3f);
        buf[4] = '\0';
        rv = 4;

    /* Error. */
    } else {
        /* "Encode" as zero bytes. */
        buf[0] = '\0';
        rv = 0;
        /* Throw exception. */
        *cex = cgul_exception__new("cgul_unicode__wctomb(): "
                                   "invalid wide character: %#0lx",
                                   (unsigned long)utf32);
        goto out;
    }

 out:
    return rv;
}

size_t
cgul_unicode__mbtohc(cgul_exception_t* cex,
                     const char* utf8,
                     size_t utf8_length,
                     size_t* index,
                     cgul_hchar_t* buf)
{
    size_t rv = 0;
    size_t i = 0;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf8);
    assert(index);
    assert(buf);

    /* Copy index into i to avoid disturbing it if an exception is thrown. */
    i = *index;

    /* Convert the character from UTF-8 to UTF-32. */
    wc = cgul_unicode__mbtowc(cex, utf8, utf8_length, &i);
    if (*cex) {
        buf[0] = CGUL_HCHAR__NUL;
        goto out;
    }

    /* Convert the character from UTF-32 to UTF-16. */
    rv = cgul_unicode__wctohc(cex, wc, buf);
    if (*cex) {
        goto out;
    }

    /* Return the new index value. */
    *index = i;

 out:
    return rv;
}

size_t
cgul_unicode__hctomb(cgul_exception_t* cex,
                     const cgul_hchar_t* utf16,
                     size_t utf16_length,
                     size_t* index,
                     char* buf)
{
    size_t rv = 0;
    size_t i = 0;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf16);
    assert(index);
    assert(buf);

    /* Copy index into i to avoid disturbing it if an exception is thrown. */
    i = *index;

    /* Convert the character from UTF-16 to UTF-32. */
    wc = cgul_unicode__hctowc(cex, utf16, utf16_length, &i);
    if (*cex) {
        goto out;
    }

    /* Convert the character from UTF-32 to UTF-8. */
    rv = cgul_unicode__wctomb(cex, wc, buf);
    if (*cex) {
        goto out;
    }

    /* Return the new index value. */
    *index = i;

 out:
    return rv;
}

cgul_wchar_t
cgul_unicode__hctowc(cgul_exception_t* cex,
                     const cgul_hchar_t* utf16,
                     size_t utf16_length,
                     size_t* index)
{
    cgul_wchar_t rv = CGUL_WCHAR__NUL;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;
    size_t i = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf16);
    assert(index);

    /*
     * See RFC 2781.
     */

    /* Copy "*index" to "i".  This way we don't alter "*index" if a
     * failure occurs. */
    i = *index;

    /* Bounds check "i". */
    if (i >= utf16_length) {
        *cex = cgul_exception__new("cgul_unicode__hctowc(): "
                                   "index out of bounds: "
                                   "%lu >= %lu",
                                   (unsigned long)i,
                                   (unsigned long)utf16_length);
        goto out;
    }

    /* In BMP below surrogate pair range. Encoded as one 16-bit value. */
    if (utf16[i] <= 0xd7ff) {
        wc = utf16[i];
        ++i;

    /* In BMP and in valid surrogate pair range for the first half. */
    } else if (utf16[i] <= 0xdbff) {
        /* Extract the first half. */
        wc = (utf16[i] - 0xd800) << 10;
        /* Increment i and bounds check. */
        if (++i >= utf16_length) {
            *cex = cgul_exception__new(
                       "cgul_unicode__hctowc(): "
                       "index out of bounds for surrogate pair: "
                       "%lu >= %lu",
                       (unsigned long)i,
                       (unsigned long)utf16_length);
            goto out;
        }
        /* Make sure the second half is in a valid range. */
        if ((utf16[i] < 0xdc00) || (utf16[i] > 0xdfff)) {
            *cex = cgul_exception__new("cgul_unicode__hctowc :"
                                       "invalid encoding");
            goto out;
        }
        /* Extract the second half. */
        wc |= (utf16[i] - 0xdc00);
        /* Add 0x10000. */
        wc += 0x10000;
        ++i;

    /* In BMP but in invalid surrogate pair range for the first half. */
    } else if (utf16[i] <= 0xdfff) {
        *cex = cgul_exception__new("cgul_unicode__hctowc :"
                                   "invalid encoding");
        goto out;

    /* In BMP above surrogate pair range.  Also encoded as one 16-bit value. */
    } else {
        wc = utf16[i];
        ++i;
    }

    /* Success */
    rv = wc;
    *index = i;

 out:
    return rv;
}

size_t
cgul_unicode__wctohc(cgul_exception_t* cex,
                     const cgul_wchar_t utf32,
                     cgul_hchar_t* buf)
{
    size_t rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(buf);

    /*
     * See RFC 2781.
     */

    /* In BMP below surrogate pair range. Encode as one 16-bit value. */
    if (utf32 <= 0xd7ff) {
        buf[0] = (cgul_hchar_t)utf32;
        buf[1] = CGUL_HCHAR__NUL;
        rv = 1;

    /* In BMP but in surrogate pair range.  Throw an exception. */
    } else if (utf32 <= 0xdfff) {
        buf[0] = CGUL_HCHAR__NUL;
        *cex = cgul_exception__new(
                   "cgul_unicode__wctohc: "
                   "character in range reserved for surrogates: %#0lx",
                   (unsigned long)utf32);
        goto out;

    /* In BMP above surrogate pair range.  Also encode as one 16-bit value. */
    } else if (utf32 <= 0xffff) {
        buf[0] = (cgul_hchar_t)utf32;
        buf[1] = CGUL_HCHAR__NUL;
        rv = 1;

    /* Above BMP.  Encode as a surrogate pair. */
    } else if (utf32 <= 0x10ffff) {
        cgul_wchar_t tmp = utf32 - 0x10000;
        buf[0] = (cgul_hchar_t)(0xd800 + (tmp >> 10));
        buf[1] = (cgul_hchar_t)(0xdc00 + (tmp & 0x3ff));
        buf[2] = CGUL_HCHAR__NUL;
        rv = 2;

    /* Error. */
    } else {
        buf[0] = CGUL_HCHAR__NUL;
        *cex = cgul_exception__new("cgul_unicode__wctohc(): "
                                   "invalid wide character: %#0lx",
                                   (unsigned long)utf32);
        goto out;
    }

 out:
    return rv;
}

size_t
cgul_unicode__get_char_count(cgul_exception_t* cex,
                             const char* utf8,
                             size_t utf8_length,
                             int skip_bom)
{
    size_t rv = 0;
    size_t rv_tmp = 0;
    size_t index = 0;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf8);

    while (index < utf8_length) {

        /* Decode the next UTF-8 character. */
        wc = cgul_unicode__mbtowc(cex, utf8, utf8_length, &index);
        if (*cex) {
            goto out;
        }

        /* Skip the byte-order mark if required. */
        if ((index == 3) && skip_bom && (wc == CGUL_WCHAR__BOM)) {
            continue;
        }

        /* Increment the character count. */
        ++rv_tmp;
    }

    /* Success */
    rv = rv_tmp;

 out:
    return rv;
}

size_t
cgul_unicode__get_hchar_count(cgul_exception_t* cex,
                              const cgul_hchar_t* utf16,
                              size_t utf16_length,
                              int skip_bom)
{
    size_t rv = 0;
    size_t rv_tmp = 0;
    size_t index = 0;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf16);

    while (index < utf16_length) {

        /* Decode the next UTF-16 character. */
        wc = cgul_unicode__hctowc(cex, utf16, utf16_length, &index);
        if (*cex) {
            goto out;
        }

        /* Skip the byte-order mark if required. */
        if ((index == 1) && skip_bom && (wc == CGUL_WCHAR__BOM)) {
            continue;
        }

        /* Increment the character count. */
        ++rv_tmp;
    }

    /* Success */
    rv = rv_tmp;

 out:
    return rv;
}

size_t
cgul_unicode__get_wchar_count(cgul_exception_t* cex,
                              const cgul_wchar_t* utf32,
                              int skip_bom)
{
    size_t rv = 0;
    size_t i = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(utf32);

    /* Iterate over each UTF-32 character. */
    for (i = 0 ; utf32[i] ; ++i) {

        /* Skip the byte-order-mark if it is the first character. */
        if ((i == 0) && skip_bom && (utf32[i] == CGUL_WCHAR__BOM)) {
            continue;
        }

        ++rv;
    }

 out:
    return rv;
}
