#include "cgul_wchar.h"

const cgul_wchar_t CGUL_WCHAR__EMPTY_STRING[1] = {0};

size_t
cgul_wchar__wcslen(cgul_exception_t* cex,
                   const cgul_wchar_t* ws)
{
    size_t rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(ws);

    /* Count all the wide characters before the null terminator. */
    while (ws[rv] != CGUL_WCHAR__NUL) {
        ++rv;
    }

 out:
    return rv;
}

void
cgul_wchar__wcscpy(cgul_exception_t* cex,
                   cgul_wchar_t* dst,
                   const cgul_wchar_t* src)
{
    size_t i = 0;
    size_t src_length = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(dst);
    assert(src);

    /* Don't do anything if dst and src point to the same memory. */
    if (dst == src) {
        goto out;
    }

    /* Determine the length of the string to copy. */
    src_length = cgul_wchar__wcslen(cex, src);

    /* Copy src to dst.  This is a little slow, but it allows "dst"
     * and "src" to overlap.  Note that we use "i <= src_length"
     * instead of the usual "i < src_length" in order to copy the null
     * terminator too. */
    for (i = 0 ; i <= src_length ; ++i) {
        dst[i] = src[i];
    }

 out:
    return;
}

int
cgul_wchar__wcscmp(const cgul_wchar_t* ws1,
                   const cgul_wchar_t* ws2)
{
    int rv = 0;

    assert(ws1);
    assert(ws2);

    /* Iterate over ws1 and ws2 searching for the first character that
     * differs. */
    for ( ; *ws1 && *ws2 ; ++ws1, ++ws2) {

        /* If this character in ws1 is less than its counterpart in
         * ws2, the ws1 string is less than the ws2 string. */
        if (*ws1 < *ws2) {
            rv = -1;
            goto out;
        }

        /* If this character in ws1 is greater than its counterpart in
         * ws2, the ws1 string is greater than the ws2 string. */
        if (*ws1 > *ws2) {
            rv = 1;
            goto out;
        }
    }

    /* If the loop above reached the end of both strings, they are equal. */
    if ((*ws1 == CGUL_WCHAR__NUL) && (*ws2 == CGUL_WCHAR__NUL)) {
        rv = 0;
        goto out;
    }

    /* If the loop above reached the end of ws1 only, ws1 is less than ws2. */
    if (*ws1 == CGUL_WCHAR__NUL) {
        rv = -1;
        goto out;
    }

    /* The loop above must have reached the end of ws2 in which case ws1
     * is greater than ws2. */
    assert(*ws2 == CGUL_WCHAR__NUL);
    rv = 1;

 out:
    return rv;
}

int
cgul_wchar__isspace(cgul_exception_t* cex,
                    cgul_wchar_t wc)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    switch (wc) {
    case (cgul_wchar_t)' ':
    case (cgul_wchar_t)'\t':
    case (cgul_wchar_t)'\n':
    case (cgul_wchar_t)'\r':
    case (cgul_wchar_t)'\f':
    case (cgul_wchar_t)'\v':
        rv = 1;
        break;
    default:
        break;
    }

 out:
    return rv;
}
