#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    char* c_as_string = NULL;
    cgul_bcd_t c = NULL;
    cgul_exception_t local = NULL;

    /* Allocate a bcd integer. */
    c = cgul_bcd__new_from_string(&local, "0xffffffffffffffff");

    /* Convert it to a decimal string.
     *
     * NOTE: This isn't really a fair comparison because the cgul_bcd
     * class does not have to do _any_ arithmetic to read in a decimal
     * number and write it back out because everything is stored as
     * binary-coded decimal.
     *
     * So, if you want to profile this class, you need to either read
     * in a hexadecimal, octal, or binary string, or you can just
     * force it to do addition, subtraction, multiplication, and
     * division.
     */
    c_as_string = cgul_bcd__as_decimal(&local, c);

    /* Print the string. */
    cgul_libc__printf(&local, "c = %s\n", c_as_string);

    {
        cgul_bcd_t quotient = cgul_bcd__new_from_string(&local, "-1234");
        cgul_bcd_t remainder = cgul_bcd__new_from_string(&local, "0");
        cgul_bcd_t dividend = cgul_bcd__new_from_string(&local, "12341368");
        cgul_bcd_t divisor = cgul_bcd__new_from_string(&local, "123418");

        /* Print the dividend and the divisor. */
        cgul_libc__printf(&local, "%12s = ", "dividend");
        cgul_bcd__print_as_decimal(&local, dividend, stdout);
        puts("");
        cgul_libc__printf(&local, "%12s = ", "divisor");
        cgul_bcd__print_as_decimal(&local, divisor, stdout);
        puts("");

        /* Divide */
        cgul_bcd__divide(&local, quotient, remainder, dividend, divisor);

        /* Print quotient and remainder. */
        cgul_libc__printf(&local, "%12s = ", "quotient");
        cgul_bcd__print_as_decimal(&local, quotient, stdout);
        puts("");
        cgul_libc__printf(&local, "%12s = ", "remainder");
        cgul_bcd__print_as_decimal(&local, remainder, stdout);
        puts("");

        if (divisor) {
            cgul_bcd__delete(divisor);
        }
        if (dividend) {
            cgul_bcd__delete(dividend);
        }
        if (remainder) {
            cgul_bcd__delete(remainder);
        }
        if (quotient) {
            cgul_bcd__delete(quotient);
        }
    }

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (c_as_string) {
        free(c_as_string);
    }
    if (c) {
        cgul_bcd__delete(c);
    }

    return rv;
}
