#include "cgul.h"

int main(int argc,
         char* argv[])
{
    int rv = 0;
    cgul_bdf_t bdf = NULL;
    cgul_bdf_glyph_t glyph = NULL;
    cgul_exception_t local = NULL;

    /* Verify the command line. */
    if (argc != 2) {
        fprintf(stderr, "Usage: main_bdf <font.bdf>\n");
        goto out;
    }

    /* Allocate a new font object. */
    bdf = cgul_bdf__new(&local);
    if (local) {
        goto out;
    }

    /* Load the font from file. */
    cgul_bdf__load_explicit(&local, bdf, argv[1], NULL);
    if(local) {
        if (local) {
            goto out;
        }
    }

#if 0
    /* Shift the font right so that no character has a negative
     * x-offset.  This is useful if you want to make sure the first
     * character on the line does not clip on the left side. */
    cgul_bdf__shift_right(&local, bdf);

    /* Some broken font drawing mechanisms ignore the x-offset
     * component of the bounding box, and draw all glyphs starting at
     * the origin.  We can compensate for this shifting the entire
     * font right (which is done above when calling
     * cgul_bdf__shift_right()) and then adding padding on the left
     * side of each glyph so that the x-offset is always zero. */
    cgul_bdf__pad_left(&local, bdf);
#endif

    /* Get the glyph for the 'A' character. */
    glyph = cgul_bdf__get_glyph(&local, bdf, 'A', 1 /* substitute */);
    if (glyph) {
        /* "Draw" the glyph on stdout. */
        cgul_bdf_glyph__fdraw(&local, glyph, stdout, NULL, NULL, '*', ' ');
    }

    /* Write the font to a temporary file. */
    cgul_libc__printf(&local, "Creating sample BDF: tmp.bdf\n");
    cgul_bdf__write(&local, bdf, "tmp.bdf");

    /* Write the font index to a temporary file. */
    cgul_libc__printf(&local, "Creating sample BDF index: tmp.bdfi\n");
    cgul_bdf__write_index(&local, bdf, "tmp.bdfi");

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    if (bdf) {
        cgul_bdf__delete(bdf);
    }

    return rv;
}
