#include <assert.h>
#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

class myobj {

public:

    /*
     * Static method that implement the cgul_cache_cxx interface.
     */

    static myobj*
    constructor(cgul_exception_t* cex)
    {
        myobj* rv = NULL;
        try {
            /* Older C++ implementations return NULL; newer ones throw
             * an exception.  The code below handles both. */
            rv = new myobj();
            if (rv == NULL) {
                throw runtime_error("out of memory");
            }
            restructor(rv);
        } catch (...) {
            /* Convert all C++ exceptions to C-style exceptions which
             * is what the underlying cgul_cache class is expecting. */
            *cex = cgul_exception__get_out_of_memory_exception();
        }
        return rv;
    }

    static void
    destructor(myobj* obj)
    {
        delete obj;
    }

    static void
    restructor(myobj* obj)
    {
        obj->x = 3;
    }

    /*
     * Instance methods.
     */

    myobj()
    {
        /* empty */
    }

    virtual ~myobj()
    {
        /* empty */
    }

    virtual int
    get_x()
    {
        return x;
    }

private:

    int x;

};

int main(int argc, char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    unsigned long int reserve = 0;
    myobj* obj = NULL;

    try {

        /* Create a new cache object. */
        cgul_cache_cxx cache((cgul_cache__constructor_t)myobj::constructor,
                             (cgul_cache__destructor_t)myobj::destructor,
                             (cgul_cache__restructor_t)myobj::restructor);

        /* Reserve 3 cache objects.  This instantiates all 3 objects and
         * reserves them (until the reserve level is reduced). */
        reserve = 3;
        cache.set_reserve(reserve);

        /* Make the reserved objects available. */
        cache.set_reserve(0);

        /* Get each reserved object. */
        for (i = 0 ; i < reserve ; ++i) {
            obj = (myobj*)cache.get();
            cout << "obj = " << obj << endl;
            cache.put(obj);
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
