/*
 * This a sample program that demonstrates how to use the cgul_crlf
 * class directly; however, for most purposes, you will probably want
 * to use the following class instead because its interface is easier
 * to use:
 *
 *     cgul_crlf_file 
 *
 * If you just need to convert a file from one EOL sequence to
 * another, the easiest method to use is
 *
 *     cgul_crlf__convert_file().
 */

#include "cgul.h"

/* Size of the block used to read from the files. */
#define BLOCK_SIZE (5)

int main(int argc, char* argv[])
{
    int rv = 0;
    int i = 0;
    unsigned long line_count = 0;
    unsigned long file_offset = 0;
    size_t rcount = 0;
    char* block = NULL;
    const char* line = NULL;
    FILE* f = NULL;
    cgul_crlf_t crlf = NULL;
    cgul_exception_t local = NULL;

    /* Verify the command line. */
    if (argc <= 1) {
        fprintf(stderr, "Usage: main_crlf <file.txt> [<file.txt> ... ]\n");
        goto out;
    }

    /* Allocate a block for reading. */
    block = (char*)cgul_libc__malloc(&local, BLOCK_SIZE);

    /* Allocate a new cgul_crlf object. */
    crlf = cgul_crlf__new(&local);

#if 0
    /* Do not strip the UTF-8 byte-order mark. */
    cgul_crlf__set_strip_utf8_bom(&local, crlf, 0);
#endif

    /* Iterate over the command line arguments. */
    for (i = 1 ; i < argc ; ++i) {

        /* Open the file. */
        f = cgul_libc__fopen(&local, argv[i], "rb");
        if (local) {
            goto out;
        }

        /* Reset the converter. */
        cgul_crlf__reset(&local, crlf, 0);

        /* Process the entire file. */
        for (;;) {

            /* Read a block from the file. */
            rcount = cgul_libc__fread(&local, block, 1, BLOCK_SIZE, f);
            if (local) {
                goto out;
            }

            /* EOF */
            if (rcount == 0) {
                break;
            }

            /* Convert the block. */
            cgul_crlf__convert(&local, crlf, block, rcount);
            if (local) {
                goto out;
            }

            /* Extract all of the lines in this block. */
            while ((line = cgul_crlf__get_line(&local, crlf)) != NULL) {
                line_count = cgul_crlf__get_line_count(&local, crlf);
                file_offset = cgul_crlf__get_line_offset(&local, crlf);
                printf("%5lu: %9lu: %s\n", line_count, file_offset, line);
            }
        }

        /* Handle the remainder if any. */
        if ((line = cgul_crlf__get_remainder(&local, crlf)) != NULL) {
            line_count = cgul_crlf__get_line_count(&local, crlf);
            file_offset = cgul_crlf__get_line_offset(&local, crlf);
            printf("%5lu: %9lu: %s\n", line_count, file_offset, line);
        }

        /* Close the file. */
        fclose(f);
        f = NULL;
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (f) {
        fclose(f);
    }
    if (block) {
        free(block);
    }
    if (crlf) {
        cgul_crlf__delete(crlf);
    }

    return rv;
}
