/*
 * This program tests the cgul_csv__quote() static method which is
 * used to quote fields before writing them to a CSV file.  Also see
 * the main_csv_sort.c program for a much more extensive example of
 * this.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_string_t row = NULL;
    cgul_exception_t local = NULL;

    /* Allocate a string to hold a row in the CSV file. */
    row = cgul_string__new(&local);

    /*
     * Create a row for a CSV file by quoting each field and
     * manually inserting the commas between fields.
     */

    /* Simple strings. */
    cgul_csv__quote(&local, "1", row);
    cgul_string__append_char(&local, row, ',');
    cgul_csv__quote(&local, "2", row);
    cgul_string__append_char(&local, row, ',');
    cgul_csv__quote(&local, "3", row);
    cgul_string__append_char(&local, row, ',');

    /* String with embedded quote. */
    cgul_csv__quote(&local, "He said, \"What?\"", row);
    cgul_string__append_char(&local, row, ',');

    /* Multi-line text. */
    cgul_csv__quote(&local, "This\nis\nmulti-line\ntext.", row);

    /* Check for errors. */
    if (local) {
        goto out;
    }

    /* Print the result. */
    cgul_libc__printf(&local, "%s\n", cgul_string__get_value(&local, row));

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (row) {
        cgul_string__delete(row);
    }

    return rv;
}
