#include "cgul.h"

static int
print_key(cgul_exception_t* cex,
          const void* key,
          void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("key = %s\n", (char*)key);
    rv = 1;

 out:
    return rv;
}

static int
print_value(cgul_exception_t* cex,
            void* value,
            void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("value = %d\n", *(int*)value);
    rv = 1;

 out:
    return rv;
}

static int
print_pair(cgul_exception_t* cex,
           const void* key,
           void* value,
           void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("key = %s  value = %d\n", (char*)key, *(int*)value);
    rv = 1;

 out:
    return rv;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    char* key = NULL;
    int* value = NULL;
    cgul_hash_t h = NULL;
    cgul_hash_node_t n = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    h = cgul_hash__new(&local);
    if (local) {
        goto out;
    }

    /* Populate the hash. */
    for (i = 0 ; i < 5 ; ++i) {
        /* Allocate a new key and value to add to the hash. */
        key = cgul_libc__calloc(&local, 1, sizeof(char) * 2);
        value = cgul_libc__calloc(&local, 1, sizeof(*value));
        if (local) {
            goto out;
        }
        /* Assign values to the newly allocated memory. */
        *key = 'a' + i;
        *value = i;
        /* Insert the key/value pair into the hash. */
        cgul_hash__insert(&local, h, key, &n);
        if (local) {
            goto out;
        }
        cgul_hash_node__set_value(&local, n, value);
    }

    /* Print the keys in forward and then reverse order. */
    cgul_hash__foldl_keys(&local, h, print_key, NULL);
    cgul_hash__foldr_keys(&local, h, print_key, NULL);

    puts("=====");

    /* Print the values in forward and then reverse order. */
    cgul_hash__foldl_values(&local, h, print_value, NULL);
    cgul_hash__foldr_values(&local, h, print_value, NULL);

    puts("=====");

    /* Print the key/value pairs in forward and then reverse order. */
    cgul_hash__foldl_pairs(&local, h, print_pair, NULL);
    cgul_hash__foldr_pairs(&local, h, print_pair, NULL);

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (h) {
        cgul_hash__free_keys(h);
        cgul_hash__free_values(h);
        cgul_hash__delete(h);
    }

    return rv;
}
