#include "cgul.h"

/**
 * Compare the integer elements <code>e1</code> and <code>e2</code>
 * returning less than zero, zero, or greater than zero if
 * <code>*e1</code> is less than, equal to, or greater than
 * <code>*e2</code> respectively.
 *
 * @param[in] e1 first element
 * @param[in] e2 second element
 * @return comparison of e1 with e2
 */
int
main__compare_integers(const void* e1,
                       const void* e2)
{
    int rv = 0;
    int int1 = *(int*)e1;
    int int2 = *(int*)e2;
    
    if (int1 < int2) {
        rv = -1;
    } else if (int1 > int2) {
        rv = 1;
    } else {
        rv = 0;
    }
    
    return rv;
}

/**
 * Push the integer <code>n</code> onto the heap <code>heap</code>.
 * If an error occurs, an exception is thrown.
 *
 * @param[in,out] cex c-style exception
 * @param[in] heap heap
 * @param[in] n integer to append
 */
void
main__push_integer(cgul_exception_t* cex,
                   cgul_heap_t heap,
                   int n)
{
    int* buf = NULL;
    
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(heap);

    /* Allocate space for a new integer. */
    buf = cgul_libc__calloc(cex, 1, sizeof(*buf));
    if (*cex) {
        goto out;
    }

    /* Push the integer onto the heap. */
    *buf = n;
    cgul_heap__push(cex, heap, buf);
    if (*cex) {
        goto out;
    }

    /* The heap now owns the integer. */
    buf = NULL;

 out:

    /* Clean up. */
    if (buf) {
        free(buf);
        buf = NULL;
    }
    
    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    int* n = 0;
    cgul_heap_t heap = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    heap = cgul_heap__new(&local, main__compare_integers);
    if (local) {
        goto out;
    }

    /* Initialize the heap. */
    for (i = 0 ; i < 10 ; ++i) {
        main__push_integer(&local, heap, i);
        if (local) {
            goto out;
        }
    }

    /* Print the top value (without removing it from the heap). */
    n = (int*)cgul_heap__top(&local, heap);
    printf("%d\n", *n);
    printf("=====\n");

    /* Print the heap. */
    while (!cgul_heap__is_empty(&local, heap)) {
        n = (int*)cgul_heap__pop(&local, heap);
        printf("%d\n", *n);
        free(n);
    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (heap) {
        cgul_heap__free_values(heap);
        cgul_heap__delete(heap);
    }

    return rv;
}
