#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

/**
 * Compare the integer elements <code>e1</code> and <code>e2</code>
 * returning less than zero, zero, or greater than zero if
 * <code>*e1</code> is less than, equal to, or greater than
 * <code>*e2</code> respectively.
 *
 * @param[in] e1 first element
 * @param[in] e2 second element
 * @return comparison of e1 with e2
 */
int
main__compare_integers(const void* e1,
                       const void* e2)
{
    int rv = 0;
    int int1 = *(int*)e1;
    int int2 = *(int*)e2;
    
    if (int1 < int2) {
        rv = -1;
    } else if (int1 > int2) {
        rv = 1;
    } else {
        rv = 0;
    }
    
    return rv;
}

/**
 * Append the integer <code>n</code> to the vector <code>v</code>.  If
 * an error occurs, an exception is thrown.
 *
 * @param[in] v vector
 * @param[in] n integer to append
 */
void
main__append_integer(cgul_vector_cxx& v,
                     int n)
{
    int* buf = NULL;

    try {

        /* Allocate space for a new integer. */
        buf = (int*)cgul_libc_cxx::calloc(1, sizeof(*buf));

        /* Append the integer to the back of the vector. */
        *buf = n;
        v.push_back(buf);

    } catch (...) {

        /* Clean up. */
        if (buf) {
            free(buf);
            buf = NULL;
        }

        /* Rethrow. */
        throw;
    }
    
    return;
}

/**
 * Print the vector of integers <code>v</code>.
 *
 * @param[in] v vector
 */
static void
main__print_vector(cgul_vector_cxx& v)
{
    int* n = NULL;
    size_t i = 0;
    size_t i_max = 0;
    
    /* Iterate over the vector. */
    i_max = v.get_size();
    for (i = 0 ; i < i_max ; ++i) {
        /* Get the next integer. */
        n = (int*)v.at(i);
        /* Print the integer. */
        printf("v[%d] = %d\n", (int)i, *n);
    }
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    int* n = NULL;
    cgul_vector_cxx v;

    try {

        /* Initialize the vector from 1 to 8 (inclusive). */
        for (i = 1 ; i < 9 ; ++i) {
            main__append_integer(v, i);
        }
        printf("===== Vector initialized. \n");
        main__print_vector(v);

        /* Heapify the vector. */
        cgul_heap_primitives_cxx::heapify(v.get_array(),
                                          v.get_size(),
                                          main__compare_integers);
        printf("===== Vector heapified. \n");
        main__print_vector(v);

        /* Append a 9 to the vector and use shift_up() to maintain heap order. */
        main__append_integer(v, 9);
        cgul_heap_primitives_cxx::shift_up(v.get_array(),
                                           v.get_size(),
                                           v.get_size() - 1,
                                           main__compare_integers);
        printf("===== Value of 9 added to heap. \n");
        main__print_vector(v);
    
        /* Replace the value of 9 with 0 and use shift_down() to maintain
         * heap order.  (Because this is a max-heap, and 9 is currently
         * the largest value on the heap, it is the first value in the
         * vector. */
        n = (int*)v.at(0);
        *n = 0;
        cgul_heap_primitives_cxx::shift_down(v.get_array(),
                                             v.get_size(),
                                             0,  /* index */
                                             0,  /* use_floyds_method */
                                             main__compare_integers);
        printf("===== Value of 0 replaced value of 9. \n");
        main__print_vector(v);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    v.free_values();

    return rv;
}
