#include <assert.h>
#include "cgul.h"

static void
print_list(cgul_exception_t* cex,
           cgul_list_t lst)
{
    cgul_list_node_t curr = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    curr = cgul_list__get_front(cex, lst);
    for ( ; curr ; curr = cgul_list_node__get_next(cex, curr)) {
        char* s = cgul_list_node__get_value(cex, curr);
        puts(s);
    }

    puts("=====");

 out:
    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_list_t lst1 = NULL;
    cgul_list_t lst2 = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    lst1 = cgul_list__new(&local);
    lst2 = cgul_list__new(&local);
    if (local) {
        goto out;
    }

    /* Insert strings into lst1 and sort. */
    cgul_list__push_back(&local, lst1, "foo");
    cgul_list__push_back(&local, lst1, "bar");
    cgul_list__push_back(&local, lst1, "baz");
    cgul_list__sort(&local, lst1, (cgul_list__compare_t)strcmp);

    /* Insert strings into lst1 and sort. */
    cgul_list__push_back(&local, lst2, "spam");
    cgul_list__push_back(&local, lst2, "eggs");
    cgul_list__push_back(&local, lst2, "baz");
    cgul_list__push_back(&local, lst2, "zzz");
    cgul_list__sort(&local, lst2, (cgul_list__compare_t)strcmp);

    /* Print the lists before merging. */
    print_list(&local, lst1);
    print_list(&local, lst2);

    /* Merge */
    cgul_list__merge(&local, lst1, lst2, (cgul_list__compare_t)strcmp);

    /* Print the list after merging. */
    print_list(&local, lst1);
    print_list(&local, lst2);

    /* Print the sizes of each list. */
    printf("lst1 size = %lu\n", cgul_list__get_size(&local, lst1));
    printf("lst2 size = %lu\n", cgul_list__get_size(&local, lst2));

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (lst2) {
        cgul_list__delete(lst2);
    }
    if (lst1) {
        cgul_list__delete(lst1);
    }

    return rv;
}
