#include <assert.h>
#include "cgul.h"

static void
print_list(cgul_exception_t* cex,
           cgul_list_t lst)
{
    cgul_list_node_t curr = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    curr = cgul_list__get_front(cex, lst);
    for ( ; curr ; curr = cgul_list_node__get_next(cex, curr)) {
        char* s = cgul_list_node__get_value(cex, curr);
        puts(s);
    }

    puts("=====");

 out:
    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_list_t lst = NULL;
    cgul_list_node_t n1 = NULL;
    cgul_list_node_t n2 = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    lst = cgul_list__new(&local);
    if (local) {
        goto out;
    }

    /* Insert static strings. */
    cgul_list__push_back(&local, lst, "foo");
    cgul_list__push_back(&local, lst, "bar");
    cgul_list__push_back(&local, lst, "baz");
    n1 = cgul_list__push_back(&local, lst, "zzz1");
    cgul_list__push_back(&local, lst, "zzz2");
    cgul_list__push_back(&local, lst, "zzz3");
    cgul_list__push_back(&local, lst, "zzz4");
    cgul_list__push_back(&local, lst, "zzz5");
    n2 = cgul_list__push_back(&local, lst, "zzz6");
    cgul_list__push_back(&local, lst, "zzz7");

    /* Print the list. */
    print_list(&local, lst);

    /* Remove a range of nodes. */
    cgul_list__remove_range(&local, lst, n1, n2, NULL);

    /* Print the list. */
    print_list(&local, lst);

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (lst) {
        cgul_list__delete(lst);
    }

    return rv;
}
