#include <assert.h>
#include "cgul.h"

static void
print_list(cgul_exception_t* cex,
           cgul_list_t lst)
{
    cgul_list_node_t curr = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    curr = cgul_list__get_front(cex, lst);
    for ( ; curr ; curr = cgul_list_node__get_next(cex, curr)) {
        char* s = cgul_list_node__get_value(cex, curr);
        puts(s);
    }

    puts("=====");

 out:
    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_list_t lst = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    lst = cgul_list__new(&local);
    if (local) {
        goto out;
    }

    /* Sort strings as they are inserted. */
    cgul_list__insert_sorted(&local, lst, "foo", (cgul_list__compare_t)strcmp);
    cgul_list__insert_sorted(&local, lst, "bar", (cgul_list__compare_t)strcmp);
    cgul_list__insert_sorted(&local, lst, "baz", (cgul_list__compare_t)strcmp);
    cgul_list__insert_sorted(&local, lst, "bar", (cgul_list__compare_t)strcmp);
    cgul_list__insert_sorted(&local, lst, "zzz", (cgul_list__compare_t)strcmp);

    /* Print list of strings. */
    print_list(&local, lst);

    /* Clear the list. */
    cgul_list__clear(&local, lst, NULL);

    /* Add unsorted strings. */
    cgul_list__push_back(&local, lst, "foo");
    cgul_list__push_back(&local, lst, "bar");
    cgul_list__push_back(&local, lst, "baz");
    cgul_list__push_back(&local, lst, "bar");
    cgul_list__push_back(&local, lst, "zzz");

    /* Sort. */
    cgul_list__sort(&local, lst, (cgul_list__compare_t)strcmp);

    /* Print list of strings. */
    print_list(&local, lst);

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (lst) {
        cgul_list__delete(lst);
    }

    return rv;
}
