#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long i = 0;
    cgul_random_lcg256_t lcg = NULL;
    cgul_big_integer_t n = NULL;
    cgul_big_integer_t max = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    lcg = cgul_random_lcg256__new(&local);
    n = cgul_big_integer__new(&local);
    max = cgul_big_integer__new_from_long(&local, 1);
    if (local) {
        goto out;
    }

    /* Limit the numbers to a maximum of 76 hex digits (or 2^304). */
    cgul_big_integer__shift_left(&local, max, max, 304);
    if (local) {
        goto out;
    }

    /* Generate numbers. */
    for (i = 0 ; i < 10 ; ++i) {
        cgul_random_lcg256__next_in_range(&local, lcg, n, max);
        cgul_big_integer__print_as_hexadecimal(&local, n, stdout);
        if (local) {
            goto out;
        }
        puts("");
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (max) {
        cgul_big_integer__delete(max);
    }
    if (n) {
        cgul_big_integer__delete(n);
    }
    if (lcg) {
        cgul_random_lcg256__delete(lcg);
    }

    return rv;
}
