#include "cgul.h"

static int
compare_pointers(char* p1,
                 char* p2)
{
    return p1 - p2;
}

static int
remove_node_cb(cgul_exception_t* cex,
               cgul_rbtree_t t,
               cgul_rbtree_node_t node,
               void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    cgul_rbtree__remove_node(cex, t, node, NULL, NULL);

    rv = 1;

 out:
    return rv;
}

static int
print_node_cb(cgul_exception_t* cex,
              cgul_rbtree_t t,
              cgul_rbtree_node_t node,
              void* data)
{
    int rv = 0;
    void* p = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    p = cgul_rbtree_node__get_key(cex, node);
    printf("%p\n", p);

    rv = 1;

 out:
    return rv;
}

int
main(int argc, char* argv[])
{
    int rv = 0;
    char* p = NULL;
    cgul_exception_t cex = NULL;
    cgul_rbtree_t t = NULL;

    t = cgul_rbtree__new(&cex, (cgul_rbtree__compare_t)&compare_pointers);
    cgul_rbtree__set_cache_size(&cex, t, 10);

    /* 1 */
    for (p = 0x0 ; p < (char*)10 ; ++p) {
        if (!cgul_rbtree__insert(&cex, t, p, NULL)) {
            cex = cgul_exception__new("cgul_rbtree__insert: failed!");
            goto out;
        }
    }

    {
        /* Remove the nodes between 0x6 and 0x8 inclusive. */
        cgul_rbtree_node_t first = NULL;
        cgul_rbtree_node_t last = NULL;
        first = cgul_rbtree__find(&cex, t, (void*)0x6);
        last = cgul_rbtree__find(&cex, t, (void*)0x8);
        if (first && last) {
            cgul_rbtree__traverse_range(&cex,
                                        t,
                                        first,
                                        last,
                                        &remove_node_cb,
                                        NULL);
        }
        /* Print remaining nodes. */
        cgul_rbtree__traverse(&cex, t, &print_node_cb, NULL);
    }

    while (cgul_rbtree__get_size(&cex, t)) {
        cgul_rbtree__remove_front(&cex, t, NULL, NULL);
    }

    /* 2 */
    for (p = 0x0 ; p < (char*)10 ; ++p) {
        if (!cgul_rbtree__insert(&cex, t, p, NULL)) {
            cex = cgul_exception__new("cgul_rbtree__insert: failed!");
            goto out;
        }
    }

    while (cgul_rbtree__get_size(&cex, t)) {
        cgul_rbtree__remove_back(&cex, t, NULL, NULL);
    }

    /* 3 -- but without the removes in order to test dtor. */
    for (p = 0x0 ; p < (char*)10 ; ++p) {
        if (!cgul_rbtree__insert(&cex, t, p, NULL)) {
            cex = cgul_exception__new("cgul_rbtree__insert: failed!");
            goto out;
        }
    }

 out:

    if (cex) {
        const char* msg = cgul_exception__get_message(cex);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(cex);
        rv = 1;
    }

    if (t) {
        cgul_rbtree__delete(t);
    }

    return rv;
}
