#include "cgul.h"

int main(int argc,
         char* argv[])
{
    int rv = 0;
    const char* line = NULL;
    size_t line_length = 0;
    cgul_string_t utf8 = NULL;
    cgul_wstring_t utf32 = NULL;
    cgul_crlf_file_t f = NULL;
    cgul_exception_t local = NULL;

    /* Create the UTF-8 and UTF-32 string objects. */
    utf8 = cgul_string__new(&local);
    utf32 = cgul_wstring__new(&local);

    /* Open the UTF-8 "tower_of_babel.txt" file. */
    f = cgul_crlf_file__new_from_fname(&local,
                                       "tower_of_babel.txt",
                                       1 /* is_block_buffered */);
    if (local) {
        goto out;
    }

    /* Do NOT strip the byte-order mark (BOM). */
    cgul_crlf_file__set_strip_utf8_bom(&local, f, 0);

    /* Iterate over each line of f. */
    while ((line = cgul_crlf_file__get_line(&local, f)) != NULL) {

        /* Get the number of bytes in "line".  Note that because line
         * is UTF-8 encoded it is not generally the case that the
         * number of characters in "line" will match the number of
         * bytes. */
        line_length = strlen(line);

        /* Convert the line from UTF-8 to UTF-32 and append it to
         * utf32.  Note that the cgul_crlf_file object also has the
         * ability to strip the leading Byte-Order Mark (BOM), but it
         * is disabled above. */
        cgul_unicode__mbstowcs(&local,
                               line,
                               line_length,
                               0, /* skip_bom */
                               utf32);

        /* Append a line break. */
        cgul_wstring__append_char(&local, utf32, '\n');
        if (local) {
            goto out;
        }
    }

    /* Convert from UTF-32 back to UTF-8.  We can then compare the
     * input and output file of the round-trip conversion. */
    cgul_unicode__wcstombs(&local,
                           cgul_wstring__get_value(&local, utf32),
                           cgul_wstring__get_length(&local, utf32),
                           0, /* skip_bom */
                           utf8);

    printf("%s", cgul_string__get_value(&local, utf8));

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (f) {
        cgul_crlf_file__delete(f);
    }
    if (utf32) {
        cgul_wstring__delete(utf32);
    }
    if (utf8) {
        cgul_string__delete(utf8);
    }

    return rv;
}
